/*
 * C
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

#if !defined TRACE_VGLITE_H
#define TRACE_VGLITE_H

// -----------------------------------------------------------------------------
// Includes
// -----------------------------------------------------------------------------

#include "trace_platform.h"

// -----------------------------------------------------------------------------
// Macros and Defines
// -----------------------------------------------------------------------------

/*
 * @brief Logs an init gpu start event
 *
 * @param[in] operation: The VGLite operation sent to the GPU
 */
#define TRACE_VGLITE_INIT_GPU_START(operation) \
	TRACE_PLATFORM_START_U32( \
		VGLITE, \
		TRACE_VGLITE_INIT_GPU, \
		TRACE_VGLITE_OP_ ## operation \
			);

/*
 * @brief Logs an init gpu end event
 */
#define TRACE_VGLITE_INIT_GPU_END() \
	TRACE_PLATFORM_END_VOID( \
		VGLITE, \
		TRACE_VGLITE_INIT_GPU \
			);

/*
 * @brief INIT_GPU event identifier
 */
#define TRACE_VGLITE_INIT_GPU 1

/*
 * @brief VGLite operation CLEAR identifier
 * This event is expected to be used to trace a call to vg_lite_clear
 */
#define TRACE_VGLITE_OP_CLEAR 1

/*
 * @brief VGLite operation DRAW (draw path) identifier
 * This event is expected to be used to trace a call to vg_lite_draw
 */
#define TRACE_VGLITE_OP_DRAW 2

/*
 * @brief VGLite operation DRAW_GRAD (draw path with gradient) identifier
 * This event is expected to be used to trace a call to vg_lite_draw_grad
 */
#define TRACE_VGLITE_OP_DRAW_GRAD 3

/*
 * @brief VGLite operation BLIT (draw image) identifier
 * This event is expected to be used to trace a call to vg_lite_blit
 */
#define TRACE_VGLITE_OP_BLIT 4

/*
 * @brief VGLite operation BLIT_RECT (draw image area) identifier
 * This event is expected to be used to trace a call to vg_lite_blit_rect
 */
#define TRACE_VGLITE_OP_BLIT_RECT 5

#endif // !defined TRACE_VGLITE_H

// -----------------------------------------------------------------------------
// EOF
// -----------------------------------------------------------------------------


