/*
 * Java
 *
 * Copyright 2021 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.demo.watch.ntp;

import java.io.IOException;

import com.microej.demo.watch.util.services.TimeService;

import ej.bon.Util;
import ej.ecom.wifi.SecurityMode;
import ej.net.util.NtpUtil;
import ej.net.util.wifi.WifiNetworkManager;
import ej.observable.KernelSimpleObservable;

/**
 * Notifies when the time has been updated.
 */
public class NetTimeService extends KernelSimpleObservable implements TimeService {

	private static final String AP_SSID = "RT595_demo"; //$NON-NLS-1$
	private static final String AP_PWD = "MicroEJ_Corp"; //$NON-NLS-1$
	private static final SecurityMode AP_SECURITY = SecurityMode.WPA2;

	private static final String NTP_SERVER = "ntp.ubuntu.com"; //$NON-NLS-1$
	private static final int NTP_PORT = 123;
	private static final int NTP_REQUEST_TIMEOUT = 5_000;

	private static final int APJOIN_TIMEOUT = 20_000;
	private static final int MAX_APJOIN_TRIES = 3;
	private static final int MAX_NTP_TRIES = 3;

	@Override
	public void start() {
		Thread thread = new Thread("NetworkTimeUpdate") { //$NON-NLS-1$
			@Override
			public void run() {
				runTask();
			}
		};
		thread.setPriority(Thread.MIN_PRIORITY);
		thread.start();
	}

	@Override
	public void stop() {
		// No need in that context
	}

	private void runTask() {
		if (checkWifiInterface() && joinWifi() && sendNtpRequest()) {
			setChanged();
			notifyObserver();
		}
	}

	private static boolean checkWifiInterface() {
		try {
			WifiNetworkManager wifiNetworkManager = new WifiNetworkManager();
			if (wifiNetworkManager.isInit()) {
				return true;
			}
		} catch (IOException | NullPointerException e) {
			// ignore exception
		}
		return false;
	}

	private static boolean joinWifi() {
		for (int i = 0; i < MAX_APJOIN_TRIES; i++) {
			try {
				System.out.println("[NetTimeService] Joining wifi AP " + AP_SSID); //$NON-NLS-1$
				WifiNetworkManager wifiNetworkManager = new WifiNetworkManager();
				wifiNetworkManager.joinAccessPoint(AP_SSID, AP_PWD, AP_SECURITY, APJOIN_TIMEOUT);
				System.out.println("[NetTimeService] Wifi AP joined"); //$NON-NLS-1$
				return true;
			} catch (IOException e) {
				// ignore exception and try again
			}
		}

		System.out.println("[NetTimeService] Could not join wifi AP"); //$NON-NLS-1$
		return false;
	}

	private static boolean sendNtpRequest() {
		for (int i = 0; i < MAX_NTP_TRIES; i++) {
			try {
				System.out.println("[NetTimeService] Sending NTP request"); //$NON-NLS-1$
				NtpUtil.updateLocalTime(NTP_SERVER, NTP_PORT, NTP_REQUEST_TIMEOUT);
				System.out.println("[NetTimeService] Updated time to " + Util.currentTimeMillis()); //$NON-NLS-1$
				return true;
			} catch (IOException e) {
				// ignore exception and try again
			}
		}

		System.out.println("[NetTimeService] NTP request failed"); //$NON-NLS-1$
		return false;
	}
}
