/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.demo.watch.util.path;

import com.microej.vivante.utils.Path;
import com.microej.vivante.utils.SvgImage;

import ej.annotation.NonNull;

/**
 * A vector graphics composed of only one path.
 */
public class SinglePathGraphics extends VectorGraphics {

	private final Path path;

	/**
	 * Creates a new vector graphics with only one path.
	 *
	 * @param path
	 *            the unique path of this vector graphics.
	 */
	public SinglePathGraphics(@NonNull Path path) {
		super(new SvgImage(new Path[] { path }));
		this.path = path;
	}

	/**
	 * Gets the path.
	 *
	 * @return the path.
	 */
	@NonNull
	protected Path getPath() {
		return this.path;
	}
}
