/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * Use of this source code is governed by a BSD-style license that can be found with this software.
 */
package com.microej.path.compass;

import com.microej.demo.watch.util.path.VectorGraphics;
import com.microej.vivante.utils.SvgImage;
import com.verisilicon.vivante.vg_lite_matrix;

/**
 * A vector path that represents the compass direction arrow.
 */
public class CompassDirection extends VectorGraphics {

	private int centerX;

	/**
	 * Creates the compass direction arrow by describing its path.
	 */
	public CompassDirection() {
		super(SvgImage.getSvgImage("compass_marker_orange.svg")); //$NON-NLS-1$
	}

	@Override
	protected void updateMatrices() {
		vg_lite_matrix matrix = getMatrix();
		vg_lite_matrix.vg_lite_identity(matrix);
		vg_lite_matrix.vg_lite_translate(this.centerX, 0, matrix);
	}

	/**
	 * Sets the compass center point coordinates.
	 *
	 * @param x
	 *            the compass center x-coordinate to set.
	 * @param y
	 *            the compass center y-coordinate to set.
	 *
	 */
	public void setCenter(int x, int y) {
		this.centerX = x;
		updateMatrices();
	}

}
