/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.selector;

import java.io.StringWriter;
import java.util.Hashtable;
import java.util.Map;

import org.w3c.css.sac.CombinatorCondition;
import org.w3c.css.sac.Condition;
import org.w3c.css.sac.ConditionalSelector;
import org.w3c.css.sac.SimpleSelector;

import com.steadystate.css.parser.selectors.AndConditionImpl;
import com.steadystate.css.parser.selectors.ConditionalSelectorImpl;

/**
 *
 */
public class AndCombinatorSelector implements Selector {

	private static final String AND_COMBINATOR_SELECTOR_VAR_PREFIX = "andCombi_";

	private static final UniversalSelector UNIVERSAL_SELECTOR = new UniversalSelector();

	private final Map<String, String> andCombinatorSelectorVars = new Hashtable<String, String>();

	@Override
	public String generate(Condition c, StringWriter writer) throws IllegalArgumentException {
		CombinatorCondition condition = this.get(c);

		String firstConditionVar = SelectorFactory.create(condition.getFirstCondition(), writer);
		String secondConditionVar = SelectorFactory.create(condition.getSecondCondition(), writer);
		StringBuilder andCombinatorSelector = new StringBuilder(firstConditionVar).append(secondConditionVar);
		String andCombinatorSelectorVar = this.andCombinatorSelectorVars.get(andCombinatorSelector.toString());
		if (andCombinatorSelectorVar != null) {
			return andCombinatorSelectorVar;
		}
		StringBuilder andCombinatorSelectorVarSb = new StringBuilder("andCombinatorSelector_").append(firstConditionVar)
				.append("_").append(secondConditionVar);
		writer.append(this.getClassName()).append(" ").append(andCombinatorSelectorVarSb);
		writer.append(" = new ");
		writer.append(this.getClassName()).append('(');
		writer.append(firstConditionVar);
		writer.append(", ");
		writer.append(secondConditionVar);
		writer.append(");\n");
		this.andCombinatorSelectorVars.put(andCombinatorSelector.toString(), andCombinatorSelectorVarSb.toString());
		return andCombinatorSelectorVarSb.toString();

	}

	private CombinatorCondition get(Condition condition) {
		if (!this.matches(condition)) {
			throw new IllegalArgumentException("Given condition is not an AndConditionImpl");
		}
		return (CombinatorCondition) condition;
	}

	@Override
	public boolean matches(Condition condition) {
		return condition instanceof AndConditionImpl;
	}

	@Override
	public String getClassName() {
		return "ej.style.selector.combinator.AndCombinator";
	}

	@Override
	public String generate(org.w3c.css.sac.Selector s, StringWriter writer) throws IllegalArgumentException {
		ConditionalSelector conditionalSelector = this.get(s);

		SimpleSelector firstSelector = conditionalSelector.getSimpleSelector();
		if (UNIVERSAL_SELECTOR.matches(firstSelector)) {
			// first selector is universal, we don't need to generate an And selector in this case, just generate
			// condition selector.
			return SelectorFactory.create(conditionalSelector.getCondition(), writer);
		}

		String simpleSelectorVar = SelectorFactory.create(conditionalSelector.getSimpleSelector(), writer);
		String conditionSelectorVar = SelectorFactory.create(conditionalSelector.getCondition(), writer);
		StringBuilder andCombinatorSelector = new StringBuilder(simpleSelectorVar).append(conditionSelectorVar);
		String andCombinatorSelectorVar = this.andCombinatorSelectorVars.get(andCombinatorSelector.toString());
		if (andCombinatorSelectorVar != null) {
			return andCombinatorSelectorVar;
		}
		StringBuilder andCombinatorSelectorVarSb = new StringBuilder(AND_COMBINATOR_SELECTOR_VAR_PREFIX)
				.append(simpleSelectorVar).append("_").append(conditionSelectorVar);
		writer.append(this.getClassName()).append(" ").append(andCombinatorSelectorVarSb);
		writer.append(" = new ");
		writer.append(this.getClassName()).append('(');
		writer.append(simpleSelectorVar);
		writer.append(", ");
		writer.append(conditionSelectorVar);
		writer.append(");\n");
		this.andCombinatorSelectorVars.put(andCombinatorSelector.toString(), andCombinatorSelectorVarSb.toString());
		return andCombinatorSelectorVarSb.toString();
	}

	private ConditionalSelector get(org.w3c.css.sac.Selector selector) {
		if (!this.matches(selector)) {
			throw new IllegalArgumentException("Given selector is not a ConditionalSelector");
		}
		return (ConditionalSelector) selector;
	}

	@Override
	public boolean matches(org.w3c.css.sac.Selector selector) {
		return selector instanceof ConditionalSelectorImpl;
	}

}
