/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.jsgenerator;

import java.util.ArrayList;
import java.util.List;

import jdk.nashorn.internal.ir.FunctionNode;
import jdk.nashorn.internal.ir.LexicalContext;
import jdk.nashorn.internal.ir.visitor.NodeVisitor;

/**
 * A whole pass on .js files to grab data such as function declaration
 */
public class JavaScriptSourcePreGenerator extends NodeVisitor<LexicalContext> {

	private final ArrayList<FunctionNode> functions;

	public JavaScriptSourcePreGenerator() {
		super(new LexicalContext());
		functions = new ArrayList<FunctionNode>();
	}

	public List<FunctionNode> getFunctions() {
		return functions;
	}

	private void addFunction(FunctionNode functionNode) {
		functions.add(functionNode);
	}

	@Override
	public boolean enterFunctionNode(FunctionNode functionNode) {
		addFunction(functionNode);
		return false; // do not analyse function of functions (only global one so far)
	}

	/**
	 * @param varName
	 * @return null if no function declared with the given name
	 */
	public FunctionNode getFunction(String varName) {
		for (FunctionNode function : functions) {
			if (function.getName().equals(varName)) {
				return function;
			}
		}
		return null;
	}
}
