/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.model.test;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.BeforeClass;
import org.junit.Ignore;

import com.is2t.giml.description.DescriptionFactory;
import com.is2t.giml.model.test.description.Node1Description;
import com.is2t.giml.model.test.description.Root1Description;
import com.is2t.giml.model.test.description.Root2Description;
import com.is2t.giml.xml.model.Attribute;
import com.is2t.giml.xml.model.Node;
import com.is2t.giml.xml.model.Template;

/**
 * The class {@code AbstractTest} provides utility methods for GIML model factory unit tests.
 */
@Ignore
public class AbstractTest {

	/**
	 * Add test descriptions to {@link DescriptionFactory}.
	 */
	@BeforeClass
	public static void fillDescriptions() {
		DescriptionFactory.addDescription("root1", new Root1Description());
		DescriptionFactory.addDescription("root2", new Root2Description());
		DescriptionFactory.addDescription("node1", new Node1Description());
	}

	/**
	 * Template tag name for tests.
	 */
	protected static final String TEMPLATE_TAG_NAME = "template";
	/**
	 * Template attribute name for tests.
	 */
	protected static final String TEMPLATE_ATTR_NAME = "name";
	/***
	 * Template parameter prefix for tests.
	 */
	protected static final String TEMPLATE_PREFIX = "{{";
	/**
	 * Template parameter suffix for tests.
	 */
	protected static final String TEMPLATE_SUFFIX = "}}";
	/**
	 * Template content tag name for tests.
	 */
	protected static final String TEMPLATE_CONTENT_TAG_NAME = "content";
	/**
	 * Template content attribute name for tests.
	 */
	protected static final String TEMPLATE_CONTENT_ATTR_NAME = "id";
	/**
	 * Template instantiation content attribute name for tests.
	 */
	protected static final String TEMPLATE_CONTENT_INSTANTIATION_ATTR_NAME = "ref";

	/**
	 * Empty set of attributes.
	 */
	protected static final Set<Attribute> NO_ATTRIBUTE = new HashSet<Attribute>();

	/**
	 * Creates a template parameter with prefix and suffix.
	 *
	 * @param name
	 *            the name of the parameter.
	 * @return a a template parameter.
	 */
	protected String createTemplateParam(String name) {
		return TEMPLATE_PREFIX + name + TEMPLATE_SUFFIX;
	}

	/**
	 * Creates an attribute set from given inputs.
	 *
	 * @param attributeDef
	 *            a name-value array of attribute to create.
	 * @return the set of attributes.
	 */
	protected static Set<Attribute> createAttributeSet(String... attributeDef) {
		Set<Attribute> attributes = new HashSet<Attribute>();
		for (int a = 0; a < attributeDef.length; a += 2) {
			attributes.add(new Attribute(attributeDef[a], attributeDef[a + 1]));
		}
		return attributes;
	}

	/**
	 * Creates a node from given inputs.
	 *
	 * @param name
	 *            the node tag name.
	 * @param attributes
	 *            the node attributes.
	 * @param children
	 *            the node children.
	 * @return the created node.
	 */
	protected static Node createNode(String name, Set<Attribute> attributes, Node... children) {
		List<Node> childNodes = new ArrayList<Node>();
		for (Node child : children) {
			childNodes.add(child);
		}
		return new Node(name, attributes, childNodes);
	}

	/**
	 * Creates a template.
	 *
	 * @param templateNode
	 *            the node containing template informations.
	 * @return a template instance.
	 */
	protected static Template createTemplate(Node templateNode) {
		return new Template(templateNode, TEMPLATE_TAG_NAME, TEMPLATE_ATTR_NAME, TEMPLATE_PREFIX, TEMPLATE_SUFFIX,
				TEMPLATE_CONTENT_TAG_NAME, TEMPLATE_CONTENT_ATTR_NAME, TEMPLATE_CONTENT_INSTANTIATION_ATTR_NAME);
	}

}
