/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.generator.test.xmlgenerator;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.junit.Assert;
import org.junit.Test;

import com.is2t.giml.xml.generator.XMLGenerator;
import com.is2t.giml.xml.generator.exception.XMLGeneratorException;
import com.is2t.giml.xml.generator.test.AbstractTest;
import com.is2t.giml.xml.model.Attribute;
import com.is2t.giml.xml.model.Document;
import com.is2t.giml.xml.model.Node;

/**
 * Unit tests for {@link XMLGenerator} class.
 */
public class XMLGenerationTest extends AbstractTest {

	@Test
	public void testSimpleXMLGeneration() throws IOException {
		Document document = new Document("doc", "root");
		XMLGenerator xmlGenerator = new XMLGenerator(document);

		File tempFile = createTempFile("temp-file.xml");
		try {
			xmlGenerator.generate(tempFile);
		} catch (XMLGeneratorException e) {
			e.printStackTrace();
		}

		System.out.println("XMLGenerationTest.testSimpleXMLGeneration() " + tempFile);
		String readFileToString = FileUtils.readFileToString(tempFile, "UTF-8");
		System.out.println(readFileToString);

		URL fileURL = AbstractTest.class.getResource("/com/is2t/giml/xml/generator/test/xmlgenerator/simple.xml");
		File referenceFile = FileUtils.toFile(fileURL);
		Assert.assertTrue(FileUtils.contentEquals(tempFile, referenceFile));
	}

	@Test
	public void testComplexXMLGeneration() throws IOException {
		List<Node> documentChildren = new ArrayList<>();
		Node emptyNode = new Node("empty");
		documentChildren.add(emptyNode);
		Set<Attribute> nodeAttributes = new HashSet<>();
		nodeAttributes.add(new Attribute("attr1", "value1"));
		nodeAttributes.add(new Attribute("attr2", "value2"));
		Node withAttributesNode = new Node("withattributes", nodeAttributes);
		documentChildren.add(withAttributesNode);
		List<Node> nodeChildren = new ArrayList<>();
		nodeChildren.add(new Node("child"));
		Node withChildrenNode = new Node("withchildren", new HashSet<Attribute>(), nodeChildren);
		documentChildren.add(withChildrenNode);
		Set<Attribute> documentAttributes = new HashSet<>();
		documentAttributes.add(new Attribute("bonjour", "hello"));
		documentAttributes.add(new Attribute("monde", "world"));
		Document document = new Document("doc", "root", documentAttributes, documentChildren);
		XMLGenerator xmlGenerator = new XMLGenerator(document);

		File tempFile = createTempFile("temp-file.xml");
		try {
			xmlGenerator.generate(tempFile);
		} catch (XMLGeneratorException e) {
			e.printStackTrace();
		}

		String readFileToString = FileUtils.readFileToString(tempFile, "UTF-8");
		System.out.println(readFileToString);

		URL fileURL = AbstractTest.class.getResource("/com/is2t/giml/xml/generator/test/xmlgenerator/complex.xml");
		File referenceFile = FileUtils.toFile(fileURL);
		Assert.assertTrue(FileUtils.contentEquals(tempFile, referenceFile));
	}

}
