/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.parser.test.xmlparser;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;

import org.apache.commons.io.FileUtils;
import org.junit.Ignore;

import com.is2t.giml.xml.parser.XMLParser;
import com.is2t.giml.xml.parser.XMLParserOptions;
import com.is2t.giml.xml.parser.test.AbstractTest;

/**
 * The class {@code AbstractXMLParserTest} provides utility methods {@link XMLParser} valid test case classes.
 */
@Ignore
public abstract class AbstractXMLParserValidTest extends AbstractTest {

	/**
	 * Gets the resource folder for tests.
	 *
	 * @return he resource folder path for tests.
	 */
	protected abstract String getResourceFolder();

	/**
	 * Gets a {@link File} instance on a resource in the current JAR.
	 *
	 * @param resourceName
	 *            the name of the resource to get, relative to the resource folder path of this test.
	 * @return the file in the JAR.
	 *
	 * @see #RESOURCE_FOLDER_PATH
	 */
	protected File getFileFromJarResource(String resourceName) {
		String resourcePath = this.getResourceFolder() + resourceName;
		URL fileURL = AbstractTest.class.getResource(resourcePath);
		return FileUtils.toFile(fileURL);
	}

	/**
	 * Gets the {@link XMLParserOptions} for the test parser.
	 *
	 * @return options for the test parser.
	 */
	protected abstract XMLParserOptions getXMLParserOptions();

	/**
	 * Gets the XML parser instance to use for tests.
	 *
	 * @param rootFile
	 *            the root file for the parser.
	 * @return an XML parser.
	 */
	protected XMLParser createXMLParser(File rootFile) throws FileNotFoundException {
		XMLParser parser = new XMLParser(rootFile);
		parser.setOptions(this.getXMLParserOptions());
		return parser;
	}

}
