/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.parser.test.xmlparser;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import com.is2t.giml.xml.parser.XMLParser;
import com.is2t.giml.xml.parser.exception.XMLParserCircularImportException;
import com.is2t.giml.xml.parser.exception.XMLParserDuplicatedDocumentException;
import com.is2t.giml.xml.parser.exception.XMLParserException;
import com.is2t.giml.xml.parser.exception.XMLParserUnknownAttributeException;
import com.is2t.giml.xml.parser.exception.XMLParserUnknownDocumentException;
import com.is2t.giml.xml.parser.test.AbstractTest;

/**
 * Test cases of {@link XMLParser} on invalid documents.
 */
public class XMLParserInvalidDocumentImportsTest extends AbstractTest {

	private static final String RESOURCE_FOLDER_PATH = "/com/is2t/giml/xml/parser/test/xmlparser/imports/invalid/";

	private static final String PARSER_OPTION_XML_EXTENSION = ".xml";
	private static final String PARSER_OPTION_DOCUMENT_ID_ATTRIBUTE_NAME = "name";
	private static final String PARSER_OPTION_IMPORT_TAGNAME = "import";
	private static final String PARSER_OPTION_IMPORT_ATTRIBUTE_NAME = "name";
	private static final boolean PARSER_OPTION_HANDLE_IMPORTS = true;

	private static File getFileFromJarResource(String resourceName) {
		String resourcePath = RESOURCE_FOLDER_PATH + resourceName;
		URL fileURL = AbstractTest.class.getResource(resourcePath);
		return FileUtils.toFile(fileURL);
	}

	private XMLParser createXMLParser(File rootFile) throws FileNotFoundException {
		XMLParser parser = new XMLParser(rootFile);
		parser.getOptions().setXMLFileExtension(PARSER_OPTION_XML_EXTENSION);
		parser.getOptions().setDocumentIdAttributeName(PARSER_OPTION_DOCUMENT_ID_ATTRIBUTE_NAME);
		parser.getOptions().setImportTagName(PARSER_OPTION_IMPORT_TAGNAME);
		parser.getOptions().setImportTagAttributeName(PARSER_OPTION_IMPORT_ATTRIBUTE_NAME);
		parser.getOptions().setHandleImports(PARSER_OPTION_HANDLE_IMPORTS);
		return parser;
	}

	@Test(expected = XMLParserException.class)
	public void testNotXMLDocument() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("notxml/notanxmlfile.xml")).parse();
	}

	@Test(expected = XMLParserException.class)
	public void testMalformedXMLDocument() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("malformed/malformedXMLDocument.xml")).parse();
	}

	@Test(expected = XMLParserUnknownAttributeException.class)
	public void testWrongXMLDocumentIDAttributeName() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("wrongrootattrname/wrongRootAttributeName.xml")).parse();
	}

	@Test(expected = XMLParserUnknownAttributeException.class)
	public void testWrongImportAttributeName() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("wrongimportattrname/wrongImportAttributeName.xml")).parse();
	}

	@Test(expected = XMLParserDuplicatedDocumentException.class)
	public void testDuplicatedDocumentIDs() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("duplicateddocs/doc1.xml")).parse();
	}

	@Test(expected = XMLParserUnknownDocumentException.class)
	public void testImportUnknowDocument() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("unknowndoc/doc1.xml")).parse();
	}

	@Test(expected = XMLParserCircularImportException.class)
	public void testCircularImportOneDocument() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("circular1/doc1.xml")).parse();
	}

	@Test(expected = XMLParserCircularImportException.class)
	public void testCircularImportMultipleDocuments() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("circularmultiple/doc1.xml")).parse();
	}

	@Test(expected = XMLParserException.class)
	public void testMisplacedImportInDocument() throws XMLParserException, FileNotFoundException {
		createXMLParser(getFileFromJarResource("misplacedimport/misplacedImport.xml")).parse();
	}

}
