package com.is2t.java.nodes;

import com.is2t.java.JavaSourceGenerator;

public class ClassDeclaration extends TypeDeclaration{
	 
	
	/**
	 * Super type (may be null when inherits from Object)
	 */
	public TypeReference superType; //Not a TypeDefinition because the superType can be a pre-existing type

	
	public ConstructorDeclaration[] constructors;

	/**
	 * Constructor used to declare an external class
	 */
	public ClassDeclaration(char[][] fullyQualifiedName){
		super( fullyQualifiedName);		
	}

	public ClassDeclaration(Modifiers modifiers, char[][] fullyQualifiedName, TypeReference superclass, TypeReference[] interfaces, ConstructorDeclaration[] constructors, MethodDeclaration[] methods, Statement[] fieldsOrInitializers){
		super( fullyQualifiedName);
		initialize( modifiers, superclass, interfaces, constructors, methods, fieldsOrInitializers);
	}
	
	public ClassDeclaration( TypeReference superclass, MethodDeclaration[] methods){
		// for innerClass
		super( new char[0][]);
		initialize( Modifiers.NoModifiers(), superclass, new TypeReference[0], new ConstructorDeclaration[0], methods, new Statement[0]);
	}
	
			
	public void initialize(Modifiers modifiers, TypeReference superclass, TypeReference[] interfaces, ConstructorDeclaration[] constructors, MethodDeclaration[] methods, Statement[] fieldsOrInitializers) {
		initialize(modifiers, interfaces, methods, fieldsOrInitializers);
		this.superType = superclass;
		this.constructors = constructors;
	}
	
	public void generateUsing(JavaSourceGenerator gen) {
		gen.generateClassDeclaration(this);
	}

	public boolean isClass() {
		return true;
	}
	 

	/**
	 * Well known types
	 */
	public static TypeDeclaration JavaLangString;
	static{
		char[] java = new char[]{'j','a','v','a'};
		char[] lang = new char[]{'l','a','n','g'};
		JavaLangString = new ClassDeclaration(new char[][]{java,lang, new char[]{'S','t','r','i','n','g'}});
	}
	
	/**
	 * search method in hierarchy
	 * Null if not found : no existing method or pre-existing method (like Object.hascode())!
	 */
	public AbstractMethodDeclaration lookForMethodAt( char[] name, TypeReference[] argsType) {
		//search on local method
		AbstractMethodDeclaration method =super.lookForMethodAt(name, argsType);	
		if ( method != null) 
			return method;
		
		//search on local constructors
		if ( constructors != null) {
			for (int i = constructors.length; --i >= 0;) {
				if ( constructors[i].equals(name, argsType))
					return  constructors[i];
			}
		}
		
		//search on hierarchy
		if ( superType != null && ! superType.isPreExistingTypeOrArray())
			return superType.declaration.lookForMethodAt( name, argsType);
		
		//not found
		return null;
	}
}
