/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2014-2018, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.is2t.support.net.natives;

import com.is2t.errors.NetErrors;

public class DatagramSocketChannelNatives {


	/**
	 * Receives data from the datagram socket associated with the file descriptor {@code fd}.
	 * <p>When this method returns, the {@code remoteHost} buffer contains the remote host address
	 * followed by the remote host port from which the data is received. Remote host port is an int.
	 * 
	 * @param fd the datagram socket file descriptor
	 * @param dst the destination buffer into which the received data will be stored
	 * @param dstOffset the offset in the destination buffer
	 * @param dstLength the maximum number of data to receive
	 * @param remoteHost the buffer into which the remote host address and port from which the data is received will be stored
	 * @param remoteHostLength the maximum number of data in the remote host buffer
	 * @param retry true when the previous call returned {@link NetErrors#J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT}
	 * and the calling process repeats the call to this operation for its completion
	 * @return long value on success, where the 32 most significant bits converted in integer is the number of data received (in bytes)
	 * and the 32 least significant bits is the remote host address size (in bytes); or a negative error code
	 * @see {@link NetErrors} for error codes
	 */
	public static native long receive(int fd, byte[] dst, int dstOffset, int dstLength, byte[] remoteHost, int remoteHostLength, boolean retry);

	/**
	 * Sends data from the datagram socket associated with the file descriptor {@code fd}
	 * to the remote host address {@code dstAddr} and {@code port}.
	 * 
	 * @param fd the datagram socket file descriptor
	 * @param data the buffer of data to send
	 * @param dataOffset the offset in the data buffer
	 * @param dataLength the number of data to send
	 * @param dstAddr the remote host IP address at which the data will be sent
	 * @param dstAddrLength the remote host address size
	 * @param dstPort the remote host port
	 * @param retry true when the previous call returned {@link NetErrors#J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT}
	 * and the calling process repeats the call to this operation for its completion
	 * @return the number of bytes sent or negative error code
	 * @see {@link NetErrors} for error codes
	 */
	public static native int send(int fd, byte[] data, int dataOffset, int dataLength, byte[] dstAddr, int dstAddrLength, int dstPort, boolean retry);

	/**
	 * Disconnects the datagram socket associated with the file descriptor {@code fd}.
	 * 
	 * @param fd the datagram socket file descriptor
	 * @param retry true when the previous call returned {@link NetErrors#J_NET_NATIVE_CODE_BLOCKED_WITHOUT_RESULT}
	 * and the calling process repeats the call to this operation for its completion
	 * @return 0 on success or negative error code
	 * @see {@link NetErrors} for error codes
	 */
	public static native int disconnect(int fd, boolean retry);

}
