/*
 * Java
 *
 * Copyright 2019-2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.verisilicon.vivante;

import java.awt.Color;

/**
 * Gradient is the class that provides vector gradient facility to Nvg_lite_gradient class.
 * <p>
 * A gradient is constructed from a matrix, an array of fractions and an array of colors.
 */
public class Gradient {

	private final Matrix matrix;

	private float[] fractions;

	private Color[] colors;

	/**
	 * Create a gradient from a matrix
	 *
	 * @param matrix
	 *            initial value of the gradient matrix.
	 *
	 **/
	public Gradient(Matrix matrix) {
		this.matrix = matrix;
		this.fractions = new float[0];
		this.colors = new Color[0];
	}

	/**
	 * Get an handle to the gradient object.
	 *
	 * @return the hashCode of the gradient object.
	 **/
	public int getHandle() {
		return this.hashCode();
	}

	/**
	 * Get the gradient matrix
	 *
	 * @return the gradient matrix
	 **/
	public Matrix getMatrix() {
		return this.matrix;
	}

	/**
	 * Get the gradient fractions
	 *
	 * @return the fractions array
	 **/
	public float[] getFractions() {
		return this.fractions;
	}

	/**
	 * Set the gradient fractions array
	 *
	 * @param intFractions
	 *            a fractions array from which the fractions array will be initialized.
	 *
	 **/
	public void setFractions(int[] intFractions) {
		this.fractions = new float[intFractions.length];

		for (int i = 0; i < intFractions.length; i++) {
			this.fractions[i] = intFractions[i] / 255f;
		}
	}

	/**
	 * Get the gradient colors
	 *
	 * @return colors a colors array from which the colors array will be initialized.
	 *
	 **/
	public Color[] getColors() {
		return this.colors;
	}

	/**
	 * Set the gradient colors array
	 *
	 * @param colors
	 *            a colors array from which the colors array will be initialized.
	 *
	 **/
	public void setColors(int[] colors) {
		this.colors = new Color[colors.length];

		for (int i = 0; i < colors.length; i++) {
			this.colors[i] = new Color(Nvg_lite.getDisplayColor(colors[i]), true);
		}
	}
}
