/*
 * Java
 *
 * Copyright 2020-2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package com.verisilicon.vivante;

import java.util.HashMap;
import java.util.Map;

/**
 * Provides access to vg_lite equivalent native APIs specific for the gradient operations.
 */
public class Nvg_lite_gradient {
	private static Map<Integer, Gradient> gradientMap = new HashMap<>();

	/**
	 * Create a Nvg_lite_gradient object.
	 *
	 * @return Nvg_lite_gradient object handle.
	 */
	public static int create() {
		// create and initialize gradient's matrix (like emb)
		int matrixHandle = Nvg_lite_matrix.create();
		Gradient m = new Gradient(Nvg_lite_matrix.get(matrixHandle));

		gradientMap.put(m.getHandle(), m);

		return m.getHandle();
	}

	/**
	 * Destroy a Nvg_lite_gradient object.
	 *
	 * @param handle
	 *            of the Nvg_lite_gradient object to destroy.
	 * @return 0
	 */
	public static int destroy(int handle) {
		Gradient gradient = get(handle);

		Nvg_lite_matrix.destroy(gradient.getMatrix().getHandle());
		gradientMap.remove(handle);

		return 0;
	}

	/**
	 * Get the Gradient handle of the Nvg_lite_gradient object.
	 *
	 * @param handle
	 *            Nvg_lite_gradient handle.
	 * @return Grandient handle.
	 */
	public static int get_matrix_handle(int handle) {
		Gradient g = get(handle);

		return g.getMatrix().getHandle();
	}

	/**
	 * Get a Gradient handle from the gradientMap.
	 *
	 * @param handle
	 *            Gradient offset in the gradientMap.
	 * @return Gradient handle.
	 */
	public static Gradient get(int handle) {
		return gradientMap.get(Integer.valueOf(handle));
	}
}
