/*
 * Java
 *
 * Copyright 2016-2020 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.ecom.wifi.natives;

import ej.ecom.wifi.SecurityMode;
import ej.ecom.wifi.WifiCapability;
import ej.ecom.wifi.WifiManager;

/**
 * An implementation of {@link WifiManager}.
 */
public class GenericWifiManagerNatives {


	/**
	 * Join a wireless network using WPA2-Personal security mode.
	 *
	 * @param ssid
	 *            the buffer into which the ssid will be stored
	 * @param ssidOffset
	 *            the offset where the ssid will be stored in the buffer
	 * @param ssidLength
	 *            the ssid buffer length
	 * @param passphrase
	 *            the buffer into which the passphrase will be stored
	 * @param passphraseOffset
	 *            the offset where the passphrase will be store in the buffer
	 * @param passphraseLength
	 *            the passphrase buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 *
	 * @deprecated Use
	 *             {@link #joinWithSecurityMode(byte[], int, int, byte[], int, int, int, boolean)}
	 *             instead. If you don't know the security mode used, use
	 *             {@link SecurityMode#UNKNOWN}.
	 */
	@Deprecated
	public static native int join(byte[] ssid, int ssidOffset, int ssidLength, byte[] passphrase, int passphraseOffset,
			int passphraseLength, boolean getResult);

	/**
	 * Join a wireless network.
	 *
	 * @param ssid
	 *            the buffer into which the ssid will be stored
	 * @param ssidOffset
	 *            the offset where the ssid will be stored in the buffer
	 * @param ssidLength
	 *            the ssid buffer length
	 * @param passphrase
	 *            the buffer into which the passphrase will be stored
	 * @param passphraseOffset
	 *            the offset where the passphrase will be store in the buffer
	 * @param passphraseLength
	 *            the passphrase buffer length
	 * @param securityModeValue
	 *            the ssid security mode value, whether {@link SecurityMode#UNKNOWN}
	 *            can be used to join the AP using any security mode is
	 *            platform-dependent.
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int joinWithSecurityMode(byte[] ssid, int ssidOffset, int ssidLength, byte[] passphrase, int passphraseOffset,
			int passphraseLength, int securityModeValue, boolean getResult);

	/**
	 * Get the state of the client side of the Wi-Fi.
	 *
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return {@link WifiClientStateNativeConstants#IDLE} or {@link WifiClientStateNativeConstants#STARTED} if success,
	 *         a negative error code if error
	 */
	public static native int getClientState(boolean getResult);

	/**
	 * Get the BSSID of the access point.
	 *
	 * @param bssid
	 *            the buffer into which the bssid will be stored (bssid is 48-bit long)
	 * @param bssidOffset
	 *            the offset where the bssid will be stored in the buffer
	 * @param bssidLength
	 *            the bssid buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int getBSSID(byte[] bssid, int bssidOffset, int bssidLength, boolean getResult);

	/**
	 * Get the channel of the access point.
	 *
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the channel if success, a negative error code if error
	 */
	public static native int getChannel(boolean getResult);

	/**
	 * Get the security mode of the access point.
	 *
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the security mode if success, a negative error code if error
	 */
	public static native int getSecurityMode(boolean getResult);

	/**
	 * Get the RSSI of the access point.
	 *
	 * @param rssi
	 *            the buffer into which the rssi will be stored (data stored on 1 byte)
	 * @param rssiOffset
	 *            the offset where the rssi will be stored in the buffer
	 * @param rssiLength
	 *            the rssi buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int getRSSI(float[] rssi, int rssiOffset, int rssiLength, boolean getResult);

	/**
	 * Get the SSID of the access point.
	 *
	 * @param ssid
	 *            the buffer into which the ssid will be stored
	 * @param ssidOffset
	 *            the offset where the ssid will be stored in the buffer
	 * @param ssidLength
	 *            the ssid buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int getSSID(byte[] ssid, int ssidOffset, int ssidLength, boolean getResult);

	/**
	 * Get the WPS modes of the access point.
	 *
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the WPS modes if success, a negative error code if error
	 */
	public static native int getWPSModes(boolean getResult);

	/**
	 * Leave wireless network.
	 *
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int leave(boolean getResult);

	/**
	 * Enable SoftAP.
	 *
	 * @param ssid
	 *            the buffer into which the ssid will be stored
	 * @param ssidOffset
	 *            the offset where the ssid will be stored in the buffer
	 * @param ssidLength
	 *            the ssid buffer length
	 * @param passphrase
	 *            passphrase
	 * @param passphraseOffset
	 *            the offset where the passphrase will be store in the buffer
	 * @param passphraseLength
	 *            the passphrase buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int enableSoftAP(byte[] ssid, int ssidOffset, int ssidLength, byte[] passphrase,
			int passphraseOffset, int passphraseLength, boolean getResult);

	/**
	 * Set Name of soft AP
	 *
	 * @param ssid
	 *            the buffer into which the ssid will be stored
	 * @param ssidOffset
	 *            the offset where the ssid will be stored in the buffer
	 * @param ssidLength
	 *            the ssid buffer length
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int setNameSoftAP(byte[] name, int nameOffset,
			int nameLength, boolean getResult);

	/**
	 * Disable SoftAP.
	 *
	 * @param getResult
	 *            false to post the request, true to get the result
	 *
	 * @return 0 if success, a negative error code if error
	 */
	public static native int disableSoftAP(boolean getResult);

	/**
	 * Return the number of Wifi Access Point currently available.
	 *
	 * @param active
	 *            true for an active scan, false for a passive scan
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return the number of access point currently available, a negative error code otherwise.
	 */
	public static native int scanAPCount(boolean active, boolean getResult);

	/**
	 * Return the ssid currently available at index (in Wifi Access Point
	 * previous listed).
	 *
	 * @param ssid
	 *            the buffer into which the ssid will be stored
	 * @param ssidOffset
	 *            the offset where the ssid will be stored in the buffer
	 * @param ssidLength
	 *            the ssid buffer length
	 * @param index
	 *            index of the Access Point currently available
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int scanAPssidAt(byte[] ssid, int ssidOffset,
			int ssidLength, int index, boolean getResult);

	/**
	 * Return the bssid currently available at index (in Wifi Access Point
	 * previous listed).
	 *
	 * @param bssid
	 *            the buffer into which the bssid will be stored
	 * @param bssidOffset
	 *            the offset where the bssid will be stored in the buffer
	 * @param bssidLength
	 *            the bssid buffer length
	 * @param index
	 *            index of the Access Point currently available
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int scanAPbssidAt(byte[] bssid, int bssidOffset,
			int bssidLength, int index, boolean getResult);

	/**
	 * Return the channel currently available at index (in Wifi Access Point
	 * previous listed).
	 *
	 * @param index
	 *            index of the Access Point currently available
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the channel if success, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int scanAPchannelAt(int index, boolean getResult);

	/**
	 * Return the rssi currently available at index (in Wifi Access Point
	 * previous listed).
	 *
	 * @param rssi
	 *            the buffer into which the rssi will be stored
	 * @param rssiOffset
	 *            the offset where the rssi will be stored in the buffer
	 * @param rssiLength
	 *            the rssi buffer length
	 * @param index
	 *            index of the Access Point currently available
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return 0 if success, a negative error code if error
	 */
	public static native int scanAPrssiAt(float[] rssi, int rssiOffset,
			int rssiLength, int index, boolean getResult);

	/**
	 * Return the mode currently available at index (in Wifi Access Point
	 * previous listed).
	 *
	 * @param index
	 *            index of the Access Point currently available
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the security mode if success, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int scanAPsecurityModeAt(int index, boolean getResult);

	/**
	 * Return the wpsModes currently available at index (in Wifi Access Point
	 * previous listed).
	 *
	 * @param index
	 *            index of the Access Point currently available
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the WPS mode if success, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int scanAPwpsModeAt(int index, boolean getResult);

	/**
	 * Return Wifi capability of the underlying hardware.
	 *
	 * @param getResult
	 *            false to post the request, true to get the result
	 * @return a positive integer representing the Wifi capability if success, or a negative error code
	 * @see {@link LLNET_ERRORS} header file for error codes
	 */
	public static native int getCapability(boolean getResult);


	public static WifiCapability convertNativeCapability(int capability){
		switch(capability){
		case WifiCapabilityNativeConstants.BOTH_EXCLUSIVE:
			return WifiCapability.BOTH_EXCLUSIVE;
		case WifiCapabilityNativeConstants.BOTH_SIMULTANEOUS:
			return WifiCapability.BOTH_SIMULTANEOUS;
		case WifiCapabilityNativeConstants.CLIENT:
			return WifiCapability.CLIENT;
		case WifiCapabilityNativeConstants.SOFT_AP:
			return WifiCapability.SOFT_AP;
		default:
			return null;
		}
	}
}
