/*
 * Copyright (c) 1998, 2013, Oracle and/or its affiliates. All rights reserved.
 * Copyright (C) 2015-2018, IS2T - EDC compliance and optimizations.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package java.security.cert;

import java.io.InputStream;


public class CertificateFactory {

	// The certificate type
	private final String type;

	// The provider implementation
	private final CertificateFactorySpi certFacSpi;

	/**
	 * Creates a CertificateFactory object of the given type, and encapsulates
	 * the given provider implementation (SPI object) in it.
	 *
	 * @param certFacSpi the provider implementation.
	 * @param type the certificate type.
	 */
	private CertificateFactory(CertificateFactorySpi certFacSpi, String type)
	{
		this.certFacSpi = certFacSpi;
		this.type = type;
	}

	public static final CertificateFactory getInstance(String type)
			throws CertificateException {
		String spiImpl = System.getProperty("CertificateFactory." + type.toUpperCase());
		try {
			CertificateFactorySpi certificateFactorySpi = (CertificateFactorySpi)Class.forName(spiImpl).newInstance();
			return new CertificateFactory(certificateFactorySpi, type);
		} catch (Exception e) {
			throw new CertificateException(type + " not found", e);
		}
	}


	public final String getType() {
		return this.type;
	}

	public final Certificate generateCertificate(InputStream inStream)
			throws CertificateException
	{
		return certFacSpi.engineGenerateCertificate(inStream);
	}

}
