
<!--
	Markdown
	
	Copyright 2016 IS2T. All rights reserved.
	IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
-->

# Description
GIML CSS Generator is a tool to generate CSS styles in Java.

This tool requires CSS rules descriptions as _jars_.  

# Supported features

## Sizes

Only fixed sizes are supported (_i.e. sizes that do not depend on DPI, screen size, ...), expect for `width` and `height` rules that can have a size value in _percent_.

> Supported size formats: px, pt, pc, in, % (for `width` and `height`)

## Colors

[Extended][extended-colors], red-green-blue and hexadecimal colors are supported.

> Supported color formats: #AAA, #AAAAAA, rgb(xxx,xxx,xxx), aliceblue, aqua, ... 

[extended-colors]: "https://www.w.org/TR/css3-color/#svg-color" "Extended Colors"

## Style

The only supported style for borders and other elements is `solid`.

> Supported styles: solid

## Selectors

All CSS selectors are supported. See [CSS selector reference](http://www.w3schools.com/cssref/css_selectors.asp "CSS selector reference") for more details.

## Rules

Following entries are supported CSS rules

*   `background-color`
*   `background-image`
*   `background-position`
*   `background-repeat`
*   `border`
*   `border-bottom`
*   `border-bottom-color`
*   `border-bottom-width`
*   `border-color`
*   `border-left`
*   `border-left-color`
*   `border-left-width`
*   `border-radius`: only same radiuses are supported, _i.e._ `border-radius` with one argument,
*   `border-right`
*   `border-right-color`
*   `border-right-width`
*   `border-top`
*   `border-top-color`
*   `border-top-width`
*   `border-width`
*   `color`
*   `font-family`
*   `font-size`
*   `font-style`
*   `font-weight`
*   `height`
*   `line-height`: use this rule to specify line height, in order to break a long text on multiple lines,
*   `margin`
*   `margin-bottom`
*   `margin-left`
*   `margin-right`
*   `margin-top`
*   `padding`
*   `padding-bottom`
*   `padding-left`
*   `padding-right`
*   `padding-top`
*   `text-align`
*   `text-decoration`: supports `underline` value. Use this rule to specify an underlined font,
*   `vertical-align`: use this rule to align text vertically,
*   `width`
* 	`word-wrap`: if not declared or if value is `none`, long text will not be break onto multiple lines. Any other value will break long text on multiple lines.

# Examples

## Declare horizontally and vertically centered text

	.center {
		text-align: center;
		vertical-align: middle; // or vertical-align: center;
	}
	
## Declare a 25px padding except for left side

	.my-padding {
		padding: 25px;
		padding-left: 0px;
	}

## Declare a centered background image

	.my-bg-image {
		background-image: url("/com.mycompany/images/myimage.png");
		background-repeat: no-repeat;
		background-position: center;
	}

## Declare a rounded background color

	.my-rounded-bg {
		background-color: black;
		border-radius: 10px;
	}

## Declare an underlined font

	.my-font {
		font-family: "Roboto";
		text-decoration: underline;
		font-size: xx-large;
	}
 


